@@ -1,8 +1,6 @@
 #' Fill in missing values with the previous or following value
 #'
 #' @description
-#' \Sexpr[results=rd, stage=render]{vctrs:::lifecycle("experimental")}
-#'
 #' `vec_fill_missing()` fills gaps of missing values with the previous or
 #' following non-missing value.
 #'
@@ -33,8 +31,10 @@
 #' df
 #'
 #' vec_fill_missing(df)
-vec_fill_missing <- function(x,
-                             direction = c("down", "up", "downup", "updown"),
-                             max_fill = NULL) {
+vec_fill_missing <- function(
+  x,
+  direction = c("down", "up", "downup", "updown"),
+  max_fill = NULL
+) {
   .Call(vctrs_fill_missing, x, direction, max_fill)
 }
