@@ -1,24 +1,11 @@
-#include "vctrs.h"
 #include "altrep-rle.h"
-#include "altrep.h"
-
-#if (!HAS_ALTREP)
-
-#include <R_ext/Rdynload.h>
-
-void vctrs_init_altrep_rle(DllInfo* dll) { }
-
-SEXP altrep_rle_Make(SEXP input) {
-  Rf_error("Need R 3.5+ for Altrep support.");
-  return R_NilValue;
-}
-
-#else
-
 
 // Initialised at load time
 R_altrep_class_t altrep_rle_class;
 
+SEXP altrep_rle_is_materialized(SEXP x) {
+  return Rf_ScalarLogical(R_altrep_data2(x) != R_NilValue);
+}
 
 SEXP altrep_rle_Make(SEXP input) {
   SEXP res = R_new_altrep(altrep_rle_class, input, R_NilValue);
@@ -51,7 +38,7 @@ Rboolean altrep_rle_Inspect(SEXP x,
                             int deep,
                             int pvec,
                             void (*inspect_subtree)(SEXP, int, int, int)) {
-  Rprintf("vctrs_rle (len=%d, materialized=%s)\n",
+  Rprintf("vctrs_altrep_rle (len=%" R_PRIdXLEN_T ", materialized=%s)\n",
           altrep_rle_Length(x),
           R_altrep_data2(x) != R_NilValue ? "T" : "F");
   return TRUE;
@@ -154,15 +141,26 @@ SEXP altrep_rle_string_Materialize(SEXP vec) {
 }
 
 void* altrep_rle_Dataptr(SEXP vec, Rboolean writeable) {
-  return STDVEC_DATAPTR(altrep_rle_string_Materialize(vec));
+  if (writeable) {
+    r_stop_internal("Can't get writeable `DATAPTR()` to `<altrep_rle>`");
+  } else {
+    // R promises not to write to this array, but we still have to return a
+    // `void*` pointer rather than a `const void*` pointer. `STRING_PTR()` is
+    // non-API so we use `STRING_PTR_RO()` and cast. This is really a bad ALTREP
+    // API. It should have been separated into `void* Dataptr()` and `const
+    // void* Dataptr_ro()`.
+    return (void*) STRING_PTR_RO(altrep_rle_string_Materialize(vec));
+  }
 }
 
 const void* altrep_rle_Dataptr_or_null(SEXP vec) {
   SEXP data2 = R_altrep_data2(vec);
-  if (data2 == R_NilValue)
-    return NULL;
 
-  return STDVEC_DATAPTR(data2);
+  if (data2 == R_NilValue) {
+    return NULL;
+  } else {
+    return r_chr_cbegin(data2);
+  }
 }
 
 
@@ -181,5 +179,3 @@ void vctrs_init_altrep_rle(DllInfo* dll) {
   // altstring
   R_set_altstring_Elt_method(altrep_rle_class, altrep_rle_string_Elt);
 }
-
-#endif // R version >= 3.5.0
