@@ -12,24 +12,24 @@ enum vctrs_class_type class_type(r_obj* x) {
     return VCTRS_CLASS_none;
   }
 
-  r_obj* class = KEEP(r_class(x));
+  r_obj* cls = KEEP(r_class(x));
 
   // Avoid corrupt objects where `x` is an object, but the class is NULL
-  if (class == r_null) {
+  if (cls == r_null) {
     FREE(1);
     return VCTRS_CLASS_none;
   }
 
-  enum vctrs_class_type type = class_type_impl(class);
+  enum vctrs_class_type type = class_type_impl(cls);
 
   FREE(1);
   return type;
 }
 
 static
-enum vctrs_class_type class_type_impl(r_obj* class) {
-  int n = r_length(class);
-  r_obj* const* p = r_chr_cbegin(class);
+enum vctrs_class_type class_type_impl(r_obj* cls) {
+  int n = r_length(cls);
+  r_obj* const* p = r_chr_cbegin(cls);
 
   // First check for bare types for which we know how many strings are
   // the classes composed of
@@ -109,13 +109,3 @@ const char* class_type_as_str(enum vctrs_class_type type) {
   }
   never_reached("class_type_as_str");
 }
-
-
-bool vec_is_partial(r_obj* x) {
-  return x == r_null || (r_typeof(x) == R_TYPE_list && r_inherits(x, "vctrs_partial"));
-}
-
-// [[ register() ]]
-r_obj* ffi_is_partial(r_obj* x) {
-  return r_lgl(vec_is_partial(x));
-}
