@@ -24,13 +24,28 @@ test_that("vec_rbind() output is tibble if any input is tibble", {
 test_that("type of column is common type of individual columns", {
   x_int <- data_frame(x = 1L)
   x_dbl <- data_frame(x = 2.5)
-  x_chr <- data_frame(x = "a")
 
   expect_equal(vec_rbind(x_int, x_int), data_frame(x = c(1L, 1L)))
   expect_equal(vec_rbind(x_int, x_dbl), data_frame(x = c(1, 2.5)))
+})
+
+test_that("incompatible columns throws common type error", {
+  x_int <- data_frame(x = 1L)
+  x_chr <- data_frame(x = "a")
 
   expect_snapshot({
-    (expect_error(vec_rbind(x_int, x_chr), class = "vctrs_error_incompatible_type"))
+    (expect_error(
+      vec_rbind(x_int, x_chr),
+      class = "vctrs_error_incompatible_type"
+    ))
+    (expect_error(
+      vec_rbind(x_int, x_chr, .error_call = call("foo")),
+      class = "vctrs_error_incompatible_type"
+    ))
+    (expect_error(
+      vec_rbind(x_int, x_chr, .ptype = x_chr, .error_call = call("foo")),
+      class = "vctrs_error_incompatible_type"
+    ))
   })
 })
 
@@ -40,7 +55,7 @@ test_that("result contains union of columns", {
       data_frame(x = 1),
       data_frame(y = 1)
     ),
-    c("x" , "y")
+    c("x", "y")
   )
 
   expect_named(
@@ -88,8 +103,14 @@ test_that("can rbind unspecified vectors", {
   expect_identical(vec_rbind(NA, df), data.frame(x = c(NA, 1)))
   expect_identical(vec_rbind(df, NA), data.frame(x = c(1, NA)))
   expect_identical(vec_rbind(NA, df, NA), data.frame(x = c(NA, 1, NA)))
-  expect_identical(vec_rbind(c(x = NA), data.frame(x = 1)), data.frame(x = c(NA, 1)))
-  expect_identical(vec_rbind(c(y = NA), df), data.frame(y = c(NA, NA), x = c(NA, 1)))
+  expect_identical(
+    vec_rbind(c(x = NA), data.frame(x = 1)),
+    data.frame(x = c(NA, 1))
+  )
+  expect_identical(
+    vec_rbind(c(y = NA), df),
+    data.frame(y = c(NA, NA), x = c(NA, 1))
+  )
 
   out <- suppressMessages(vec_rbind(c(x = NA, x = NA), df))
   exp <- data.frame(x...1 = c(NA, NA), x...2 = c(NA, NA), x = c(NA, 1))
@@ -99,8 +120,14 @@ test_that("can rbind unspecified vectors", {
 test_that("as_df_row() tidies the names of unspecified vectors", {
   expect_identical(as_df_row(c(NA, NA)), c(NA, NA))
   expect_identical(as_df_row(unspecified(2)), unspecified(2))
-  expect_identical(as_df_row(c(a = NA, a = NA), quiet = TRUE), data.frame(a...1 = NA, a...2 = NA))
-  expect_identical(as_df_row(c(a = TRUE, a = TRUE), quiet = TRUE), data.frame(a...1 = TRUE, a...2 = TRUE))
+  expect_identical(
+    as_df_row(c(a = NA, a = NA), quiet = TRUE),
+    data.frame(a...1 = NA, a...2 = NA)
+  )
+  expect_identical(
+    as_df_row(c(a = TRUE, a = TRUE), quiet = TRUE),
+    data.frame(a...1 = TRUE, a...2 = TRUE)
+  )
 })
 
 test_that("can rbind spliced lists", {
@@ -165,31 +192,71 @@ test_that("can rbind table objects (#913)", {
 
 test_that("can rbind missing vectors", {
   expect_identical(vec_rbind(c(x = na_int)), data_frame(x = na_int))
-  expect_identical(vec_rbind(c(x = na_int), c(x = na_int)), data_frame(x = int(na_int, na_int)))
+  expect_identical(
+    vec_rbind(c(x = na_int), c(x = na_int)),
+    data_frame(x = int(na_int, na_int))
+  )
 })
 
 test_that("vec_rbind() respects size invariants (#286)", {
   expect_identical(vec_rbind(), new_data_frame(n = 0L))
 
   expect_identical(vec_rbind(int(), int()), new_data_frame(n = 2L))
-  expect_identical(vec_rbind(c(x = int()), c(x = TRUE)), new_data_frame(list(x = lgl(NA, TRUE))))
+  expect_identical(
+    vec_rbind(c(x = int()), c(x = TRUE)),
+    new_data_frame(list(x = lgl(NA, TRUE)))
+  )
 
-  expect_identical(vec_rbind(int(), new_data_frame(n = 2L), int()), new_data_frame(n = 4L))
+  expect_identical(
+    vec_rbind(int(), new_data_frame(n = 2L), int()),
+    new_data_frame(n = 4L)
+  )
 })
 
 test_that("can repair names in `vec_rbind()` (#229)", {
   expect_snapshot({
     (expect_error(vec_rbind(.name_repair = "none"), "can't be `\"none\"`"))
-    (expect_error(vec_rbind(.name_repair = "minimal"), "can't be `\"minimal\"`"))
-    (expect_error(vec_rbind(list(a = 1, a = 2), .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique"))
+    (expect_error(
+      vec_rbind(.name_repair = "minimal"),
+      "can't be `\"minimal\"`"
+    ))
+    (expect_error(
+      vec_rbind(list(a = 1, a = 2), .name_repair = "check_unique"),
+      class = "vctrs_error_names_must_be_unique"
+    ))
   })
 
-  expect_named(vec_rbind(list(a = 1, a = 2), .name_repair = "unique"), c("a...1", "a...2"))
+  expect_named(
+    vec_rbind(list(a = 1, a = 2), .name_repair = "unique"),
+    c("a...1", "a...2")
+  )
 
   expect_named(vec_rbind(list(`_` = 1)), "_")
   expect_named(vec_rbind(list(`_` = 1), .name_repair = "universal"), c("._"))
 
-  expect_named(vec_rbind(list(a = 1, a = 2), .name_repair = ~ toupper(.)), c("A", "A"))
+  expect_named(
+    vec_rbind(list(a = 1, a = 2), .name_repair = ~ toupper(.)),
+    c("A", "A")
+  )
+})
+
+test_that("can repair names quietly", {
+  local_name_repair_verbose()
+
+  expect_snapshot({
+    res_unique <- vec_rbind(
+      c(x = 1, x = 2),
+      c(x = 3, x = 4),
+      .name_repair = "unique_quiet"
+    )
+    res_universal <- vec_rbind(
+      c("if" = 1, "in" = 2),
+      c("if" = 3, "for" = 4),
+      .name_repair = "universal_quiet"
+    )
+  })
+  expect_named(res_unique, c("x...1", "x...2"))
+  expect_named(res_universal, c(".if", ".in", ".for"))
 })
 
 test_that("can construct an id column", {
@@ -205,7 +272,8 @@ test_that("can construct an id column", {
 
 test_that("vec_rbind() fails with arrays of dimensionality > 3", {
   expect_snapshot({
-    (expect_error(vec_rbind(array(NA, c(1, 1, 1))), "Can't bind arrays"))
+    (expect_error(vec_rbind(array(NA, c(1, 1, 1)))))
+    (expect_error(vec_rbind(array(NA, c(1, 1, 1)), .error_call = call("foo"))))
   })
 })
 
@@ -286,6 +354,9 @@ test_that("vec_rbind() takes the proxy and restores", {
     vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
       x
     },
+    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) {
+      x
+    },
     vec_proxy.vctrs_foobar = function(x, ...) {
       x
     },
@@ -314,6 +385,9 @@ test_that("vec_rbind() proxies before initializing", {
     vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
       x
     },
+    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) {
+      x
+    },
     vec_proxy.vctrs_foobar = function(x, ...) {
       new_data_frame(x)
     },
@@ -336,10 +410,25 @@ test_that("vec_rbind() requires a data frame proxy for data frame ptypes", {
 
   local_methods(
     vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) x,
+    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) x,
     vec_proxy.vctrs_foobar = function(x, ...) 1
   )
 
-  expect_error(vec_rbind(df, df), "Attempt to restore data frame from a double")
+  expect_error(
+    vec_rbind(df, df),
+    "Can't fill a data frame that doesn't have a data frame proxy"
+  )
+})
+
+test_that("names of `...` are used for type and cast errors even when zapped", {
+  xs <- list(a = data_frame(x = 1), b = data_frame(x = "2"))
+
+  expect_snapshot(error = TRUE, {
+    vec_rbind(!!!xs)
+  })
+  expect_snapshot(error = TRUE, {
+    vec_rbind(!!!xs, .ptype = data_frame(x = double()))
+  })
 })
 
 test_that("monitoring: name repair while rbinding doesn't modify in place", {
@@ -380,7 +469,10 @@ test_that("performance: Row binding with df-cols doesn't duplicate on every assi
 test_that("vec_cbind() reports error context", {
   expect_snapshot({
     (expect_error(vec_cbind(foobar(list()))))
+    (expect_error(vec_cbind(foobar(list()), .error_call = call("foo"))))
+
     (expect_error(vec_cbind(a = 1:2, b = int())))
+    (expect_error(vec_cbind(a = 1:2, b = int(), .error_call = call("foo"))))
   })
 })
 
@@ -426,7 +518,10 @@ test_that("duplicate names are de-deduplicated", {
 
   expect_snapshot({
     (expect_named(vec_cbind(x = 1, x = 1), c("x...1", "x...2")))
-    (expect_named(vec_cbind(data.frame(x = 1), data.frame(x = 1)), c("x...1", "x...2")))
+    (expect_named(
+      vec_cbind(data.frame(x = 1), data.frame(x = 1)),
+      c("x...1", "x...2")
+    ))
   })
 })
 
@@ -462,11 +557,20 @@ test_that("can override default .nrow", {
 
 test_that("can repair names in `vec_cbind()` (#227)", {
   expect_snapshot({
-    (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "none"), "can't be `\"none\"`"))
-    (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique"))
+    (expect_error(
+      vec_cbind(a = 1, a = 2, .name_repair = "none"),
+      "can't be `\"none\"`"
+    ))
+    (expect_error(
+      vec_cbind(a = 1, a = 2, .name_repair = "check_unique"),
+      class = "vctrs_error_names_must_be_unique"
+    ))
   })
 
-  expect_named(vec_cbind(a = 1, a = 2, .name_repair = "unique"), c("a...1", "a...2"))
+  expect_named(
+    vec_cbind(a = 1, a = 2, .name_repair = "unique"),
+    c("a...1", "a...2")
+  )
 
   expect_named(vec_cbind(`_` = 1, .name_repair = "universal"), "._")
 
@@ -474,16 +578,42 @@ test_that("can repair names in `vec_cbind()` (#227)", {
   expect_named(vec_cbind(a = 1, a = 2, .name_repair = toupper), c("A", "A"))
 })
 
+test_that("can repair names quietly", {
+  local_name_repair_verbose()
+
+  expect_snapshot({
+    res_unique <- vec_cbind(x = 1, x = 2, .name_repair = "unique_quiet")
+    res_universal <- vec_cbind(
+      "if" = 1,
+      "in" = 2,
+      .name_repair = "universal_quiet"
+    )
+  })
+  expect_named(res_unique, c("x...1", "x...2"))
+  expect_named(res_universal, c(".if", ".in"))
+})
+
 test_that("can supply `.names_to` to `vec_rbind()` (#229)", {
-  expect_error(vec_rbind(.names_to = letters), "must be")
-  expect_error(vec_rbind(.names_to = 10), "must be")
+  expect_snapshot({
+    (expect_error(vec_rbind(data_frame(), .names_to = letters)))
+    (expect_error(vec_rbind(data_frame(), .names_to = 10)))
+    (expect_error(vec_rbind(
+      data_frame(),
+      .names_to = letters,
+      .error_call = call("foo")
+    )))
+  })
 
   x <- data_frame(foo = 1:2, bar = 3:4)
   y <- data_frame(foo = 5L, bar = 6L)
 
   expect_identical(
     vec_rbind(a = x, b = y, .names_to = "quux"),
-    data_frame(quux = c("a", "a", "b"), foo = c(1L, 2L, 5L), bar = c(3L, 4L, 6L))
+    data_frame(
+      quux = c("a", "a", "b"),
+      foo = c(1L, 2L, 5L),
+      bar = c(3L, 4L, 6L)
+    )
   )
   expect_identical(
     vec_rbind(a = x, b = y, .names_to = "foo"),
@@ -525,7 +655,10 @@ test_that("vec_cbind() returns visibly (#452)", {
 
 test_that("vec_cbind() packs named data frames (#446)", {
   expect_identical(vec_cbind(data_frame(y = 1:3)), data_frame(y = 1:3))
-  expect_identical(vec_cbind(x = data_frame(y = 1:3)), data_frame(x = data_frame(y = 1:3)))
+  expect_identical(
+    vec_cbind(x = data_frame(y = 1:3)),
+    data_frame(x = data_frame(y = 1:3))
+  )
 })
 
 test_that("vec_cbind() packs 1d arrays", {
@@ -572,8 +705,12 @@ test_that("names are not repaired if packed", {
 
 test_that("vec_cbind() fails with arrays of dimensionality > 3", {
   a <- array(NA, c(1, 1, 1))
-  expect_error(vec_cbind(a), "Can't bind arrays")
-  expect_error(vec_cbind(x = a), "Can't bind arrays")
+
+  expect_snapshot({
+    (expect_error(vec_cbind(a)))
+    (expect_error(vec_cbind(a, .error_call = call("foo"))))
+    (expect_error(vec_cbind(x = a)))
+  })
 })
 
 test_that("monitoring: name repair while cbinding doesn't modify in place", {
@@ -720,10 +857,21 @@ test_that("rbind repairs names of data frames (#704)", {
   expect_identical(vec_rbind(df), df_repaired)
   expect_identical(vec_rbind(df, df), vec_rbind(df_repaired, df_repaired))
 
-  expect_error(
-    vec_rbind(df, df, .name_repair = "check_unique"),
-    class = "vctrs_error_names_must_be_unique"
-  )
+  expect_snapshot({
+    (expect_error(
+      vec_rbind(df, df, .name_repair = "check_unique"),
+      class = "vctrs_error_names_must_be_unique"
+    ))
+    (expect_error(
+      vec_rbind(
+        df,
+        df,
+        .name_repair = "check_unique",
+        .error_call = call("foo")
+      ),
+      class = "vctrs_error_names_must_be_unique"
+    ))
+  })
 })
 
 test_that("vec_rbind() works with simple homogeneous foreign S3 classes", {
@@ -734,8 +882,6 @@ test_that("vec_rbind() works with simple homogeneous foreign S3 classes", {
 })
 
 test_that("vec_rbind() works with simple homogeneous foreign S4 classes", {
-  skip_if_cant_set_names_on_s4()
-
   joe1 <- .Counts(1L, name = "Joe")
   joe2 <- .Counts(2L, name = "Joe")
 
@@ -758,12 +904,13 @@ test_that("vec_rbind() fails with complex foreign S3 classes", {
 })
 
 test_that("vec_rbind() fails with complex foreign S4 classes", {
-  skip_if_cant_set_names_on_s4()
-
   expect_snapshot({
     joe <- .Counts(1L, name = "Joe")
     jane <- .Counts(2L, name = "Jane")
-    (expect_error(vec_rbind(set_names(joe, "x"), set_names(jane, "y")), class = "vctrs_error_incompatible_type"))
+    (expect_error(
+      vec_rbind(set_names(joe, "x"), set_names(jane, "y")),
+      class = "vctrs_error_incompatible_type"
+    ))
   })
 })
 
@@ -775,63 +922,121 @@ test_that("vec_rbind() falls back to c() if S3 method is available", {
   y_df <- data_frame(x = y)
 
   expect_error(vec_rbind(x_df, y_df), class = "vctrs_error_incompatible_type")
+  expect_error(vec_c(x_df, y_df), class = "vctrs_error_incompatible_type")
+  expect_error(
+    list_unchop(list(x_df, y_df), indices = list(1, 2)),
+    class = "vctrs_error_incompatible_type"
+  )
 
-  out <- with_methods(
-    c.vctrs_foobar = function(...) quux(NextMethod()),
-    vec_rbind(x_df, y_df)
+  with_c_method <- function(expr) {
+    with_methods(
+      c.vctrs_foobar = function(...) quux(NextMethod()),
+      expr
+    )
+  }
+
+  out <- with_c_method(vec_rbind(x_df, y_df))
+  exp <- data_frame(x = quux(c(1, 2)))
+  expect_identical(out, exp)
+
+  expect_identical(with_c_method(vec_c(x_df, y_df)), exp)
+  expect_identical(
+    with_c_method(list_unchop(list(x_df, y_df), indices = list(1, 2))),
+    exp
   )
-  expect_identical(out, data_frame(x = quux(c(1, 2))))
 
   # Fallback is used with data frame subclasses, with or without
   # ptype2 method
   foo_df <- foobaz(x_df)
   bar_df <- foobaz(y_df)
 
-  out <- with_methods(
-    c.vctrs_foobar = function(...) quux(NextMethod()),
-    vec_rbind(foo_df, bar_df)
-  )
-  expect_identical(out, foobaz(data_frame(x = quux(c(1, 2)))))
+  out <- with_c_method(vec_rbind(foo_df, bar_df))
+  exp <- foobaz(data_frame(x = quux(c(1, 2))))
+  expect_identical(out, exp)
 
-  out <- with_methods(
-    c.vctrs_foobar = function(...) quux(NextMethod()),
-    vec_ptype2.vctrs_foobaz.vctrs_foobaz = function(...) foobaz(df_ptype2(...)),
-    vec_rbind(foo_df, bar_df)
+  expect_identical(with_c_method(vec_c(foo_df, bar_df)), exp)
+  expect_identical(
+    with_c_method(list_unchop(list(foo_df, bar_df), indices = list(1, 2))),
+    exp
   )
-  expect_identical(out, foobaz(data_frame(x = quux(c(1, 2)))))
 
-  skip("FIXME: c() fallback with recursion through df-col")
+  with_hybrid_methods <- function(expr, cast = TRUE) {
+    methods <- list(
+      c.vctrs_foobar = function(...) quux(NextMethod()),
+      vec_ptype2.vctrs_foobaz.vctrs_foobaz = function(...) {
+        foobaz(df_ptype2(...))
+      },
+      vec_cast.vctrs_foobaz.vctrs_foobaz = if (cast) {
+        function(...) foobaz(df_cast(...))
+      }
+    )
+    with_methods(expr, !!!compact(methods))
+  }
+
+  expect_equal(
+    with_hybrid_methods(
+      cast = FALSE,
+      vec_rbind(foo_df, bar_df)
+    ),
+    foobaz(data_frame(x = quux(c(1, 2))))
+  )
+
+  # Falls back to data frame because there is no ptype2/cast methods
+  out <- with_hybrid_methods(vec_rbind(foo_df, bar_df))
+  exp <- foobaz(data_frame(x = quux(c(1, 2))))
+  expect_identical(out, exp)
+
+  expect_identical(with_hybrid_methods(vec_c(foo_df, bar_df)), exp)
+  expect_identical(
+    with_hybrid_methods(list_unchop(
+      list(foo_df, bar_df),
+      indices = list(1, 2)
+    )),
+    exp
+  )
 
   wrapper_x_df <- data_frame(x = x_df)
   wrapper_y_df <- data_frame(x = y_df)
 
-  out <- with_methods(
-    c.vctrs_foobar = function(...) quux(NextMethod()),
-    vec_rbind(wrapper_x_df, wrapper_y_df)
+  out <- with_c_method(vec_rbind(wrapper_x_df, wrapper_y_df))
+  exp <- data_frame(x = data_frame(x = quux(c(1, 2))))
+  expect_identical(out, exp)
+
+  expect_identical(with_c_method(vec_c(wrapper_x_df, wrapper_y_df)), exp)
+  expect_identical(
+    with_c_method(list_unchop(
+      list(wrapper_x_df, wrapper_y_df),
+      indices = list(1, 2)
+    )),
+    exp
   )
-  expect_identical(out, data_frame(data_frame(x = quux(c(1, 2)))))
 })
 
 test_that("c() fallback works with unspecified columns", {
   local_methods(
     c.vctrs_foobar = function(...) foobar(NextMethod()),
-    `[.vctrs_foobar` = function(x, i, ...) foobar(NextMethod(), dispatched = TRUE)
+    `[.vctrs_foobar` = function(x, i, ...) {
+      foobar(NextMethod(), dispatched = TRUE)
+    }
   )
 
   out <- vec_rbind(
     data_frame(x = foobar(1)),
     data_frame(y = foobar(2))
   )
-  expect_identical(out, data_frame(
-    x = foobar(c(1, NA), dispatched = TRUE),
-    y = foobar(c(NA, 2), dispatched = TRUE)
-  ))
+  expect_identical(
+    out,
+    data_frame(
+      x = foobar(c(1, NA), dispatched = TRUE),
+      y = foobar(c(NA, 2), dispatched = TRUE)
+    )
+  )
 })
 
 test_that("c() fallback works with vctrs-powered data frame subclass", {
   local_methods(
-    c.vctrs_quux = function(...) quux(NextMethod(), c_dispatched = TRUE),
-    `[.vctrs_quux` = function(x, i, ...) quux(NextMethod(), bracket_dispatched = TRUE)
+    c.vctrs_quux = function(...) quux(vec_paste0(NextMethod(), "-c")),
+    `[.vctrs_quux` = function(x, i, ...) quux(vec_paste0(NextMethod(), "-["))
   )
   local_foobar_df_methods()
 
@@ -840,11 +1045,11 @@ test_that("c() fallback works with vctrs-powered data frame subclass", {
   df2 <- data_frame(x = quux(4:5))
 
   out <- vctrs::vec_rbind(df1, df2)
-  exp <- foobar(data_frame(x = quux(1:5, c_dispatched = TRUE)))
+  exp <- foobar(data_frame(x = quux(paste0(1:5, "-c"))))
   expect_identical(out, exp)
 
   out <- vctrs::vec_rbind(df2, df1)
-  exp <- foobar(data_frame(x = quux(c(4:5, 1:3), c_dispatched = TRUE)))
+  exp <- foobar(data_frame(x = quux(paste0(c(4:5, 1:3), "-c"))))
   expect_identical(out, exp)
 
   ### Disjoint case
@@ -853,7 +1058,7 @@ test_that("c() fallback works with vctrs-powered data frame subclass", {
 
   out <- vctrs::vec_rbind(df1, df2)
   exp <- foobar(data_frame(
-    x = quux(c(1:3, NA, NA), bracket_dispatched = TRUE),
+    x = quux(c(paste0(1:3, "-c-["), paste0(c(NA, NA), "-["))),
     y = c(rep(NA, 3), 4:5)
   ))
   expect_identical(out, exp)
@@ -861,7 +1066,7 @@ test_that("c() fallback works with vctrs-powered data frame subclass", {
   out <- vctrs::vec_rbind(df2, df1)
   exp <- foobar(data_frame(
     y = c(4:5, rep(NA, 3)),
-    x = quux(c(NA, NA, 1:3), bracket_dispatched = TRUE)
+    x = quux(c(paste0(c(NA, NA), "-["), paste0(1:3, "-c-[")))
   ))
   expect_identical(out, exp)
 })
@@ -880,10 +1085,6 @@ test_that("vec_rbind() falls back to c() if S3 method is available for S4 class"
 })
 
 test_that("rbind supports names and inner names (#689)", {
-  skip_if(getRversion() >= "4.1.0", "work around r-devel bug")
-  # Introduced in
-  # https://github.com/wch/r-source/commit/275bb3db02491899bbadc28fea69dcdd6fedf41e
-
   out <- vec_rbind(
     data_frame(x = list(a = 1, b = 2)),
     data_frame(x = list(3)),
@@ -893,7 +1094,10 @@ test_that("rbind supports names and inner names (#689)", {
 
   vec_x <- set_names(1:3, letters[1:3])
   vec_y <- c(FOO = 4L)
-  oo_x <- set_names(as.POSIXlt(c("2020-01-01", "2020-01-02", "2020-01-03")), letters[1:3])
+  oo_x <- set_names(
+    as.POSIXlt(c("2020-01-01", "2020-01-02", "2020-01-03")),
+    letters[1:3]
+  )
   oo_y <- c(FOO = as.POSIXlt(c("2020-01-04")))
   df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
   df_y <- new_data_frame(list(x = 4L), row.names = "d")
@@ -917,7 +1121,9 @@ test_that("rbind supports names and inner names (#689)", {
 })
 
 test_that("vec_rbind() doesn't fall back to c() with proxied classes (#1119)", {
-  foobar_rcrd <- function(x, y) new_rcrd(list(x = x, y = y), class = "vctrs_foobar")
+  foobar_rcrd <- function(x, y) {
+    new_rcrd(list(x = x, y = y), class = "vctrs_foobar")
+  }
 
   x <- foobar_rcrd(x = 1:2, y = 3:4)
 
@@ -956,18 +1162,65 @@ test_that("vec_rbind() zaps names when name-spec is zap() and names-to is NULL",
   )
 })
 
-test_that("can't zap names when `.names_to` is supplied", {
+test_that("can zap names even when `.names_to` is supplied", {
   expect_identical(
     vec_rbind(foo = c(x = 1), .names_to = zap(), .name_spec = zap()),
     data.frame(x = 1)
   )
-  expect_snapshot({
-    (expect_error(
-      vec_rbind(foo = c(x = 1), .names_to = "id", .name_spec = zap()),
-      "Can't zap outer names when `.names_to` is supplied.",
-      fixed = TRUE
-    ))
-  })
+  expect_identical(
+    vec_rbind(
+      foo = data.frame(x = 1, row.names = "row"),
+      .names_to = zap(),
+      .name_spec = zap()
+    ),
+    data.frame(x = 1)
+  )
+
+  # We previously didn't allow `.name_spec = zap()` when `.names_to = "id"`,
+  # but this does have a use case - zapping inner row names while also moving
+  # outer names into a new column
+  expect_identical(
+    vec_rbind(foo = c(x = 1), .names_to = "id", .name_spec = zap()),
+    data.frame(id = "foo", x = 1)
+  )
+  expect_identical(
+    vec_rbind(
+      foo = data.frame(x = 1, row.names = "row"),
+      .names_to = "id",
+      .name_spec = zap()
+    ),
+    data.frame(id = "foo", x = 1)
+  )
+})
+
+test_that("can request 'inner' names when `.names_to` is supplied", {
+  # Note how it can be useful to lock `.name_spec` to `"inner"` in your API, but
+  # still expose `.names_to` to your users and allow all of its options.
+  # `purrr::list_rbind()` does this.
+  expect_identical(
+    vec_rbind(
+      foo = data.frame(x = 1, row.names = "row"),
+      .names_to = zap(),
+      .name_spec = "inner"
+    ),
+    data.frame(x = 1, row.names = "row")
+  )
+  expect_identical(
+    vec_rbind(
+      foo = data.frame(x = 1, row.names = "row"),
+      .names_to = "id",
+      .name_spec = "inner"
+    ),
+    data.frame(id = "foo", x = 1, row.names = "row")
+  )
+  expect_identical(
+    vec_rbind(
+      foo = data.frame(x = 1, row.names = "row"),
+      .names_to = NULL,
+      .name_spec = "inner"
+    ),
+    data.frame(x = 1, row.names = "row")
+  )
 })
 
 test_that("can zap outer names from a name-spec (#1215)", {
@@ -983,6 +1236,16 @@ test_that("can zap outer names from a name-spec (#1215)", {
     vec_names(vec_rbind(a = df, df_named, .name_spec = zap_outer_spec)),
     c("...1", "...2", "foo")
   )
+
+  # These days it is more efficient to use a name-spec of "inner" (#1988)
+  expect_identical(
+    vec_rbind(a = df, .names_to = NULL, .name_spec = zap_outer_spec),
+    vec_rbind(a = df, .names_to = NULL, .name_spec = "inner")
+  )
+  expect_identical(
+    vec_rbind(a = df, df_named, .name_spec = zap_outer_spec),
+    vec_rbind(a = df, df_named, .name_spec = "inner")
+  )
 })
 
 test_that("column names are treated consistently in vec_rbind()", {
@@ -1001,6 +1264,79 @@ test_that("column names are treated consistently in vec_rbind()", {
   expect_error(vec_rbind(x, x), "Can't bind arrays")
 })
 
+test_that("can repair names of row-binded vectors (#1567)", {
+  local_name_repair_verbose()
+  expect_silent(
+    expect_named(
+      vec_rbind(
+        x = 1:3,
+        y = 4:6,
+        .name_repair = function(x) c("a", "a", "a")
+      ),
+      c("a", "a", "a")
+    )
+  )
+})
+
+test_that("can repair names of row-binded matrices", {
+  local_name_repair_verbose()
+  expect_silent({
+    expect_named(
+      vec_rbind(
+        x = matrix(1:3, 1),
+        y = matrix(4:6, 1),
+        .name_repair = function(x) c("a", "a", "a")
+      ),
+      c("a", "a", "a")
+    )
+
+    expect_named(
+      vec_rbind(
+        x = matrix(1:3, 1),
+        y = 4:6,
+        .name_repair = function(x) c("a", "a", "a")
+      ),
+      c("a", "a", "a")
+    )
+  })
+})
+
+test_that("vec_rbind() only restores one time", {
+  restored <- list()
+
+  local_methods(
+    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) x,
+    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) x,
+    vec_proxy.vctrs_foobar = function(x, ...) x,
+    vec_restore.vctrs_foobar = function(x, to, ...) {
+      # Ignore proxying and restoration of ptypes
+      if (length(x)) {
+        restored <<- c(restored, list(x))
+      }
+      foobar(x)
+    }
+  )
+
+  df <- data_frame(x = foobar(1:3))
+  vec_rbind(df, df)
+
+  expect_equal(
+    restored,
+    list(
+      rep(na_int, 6), # From `vec_init()`
+      foobar(c(1:3, 1:3)) # Final restoration
+    )
+  )
+})
+
+test_that("vec_rbind() applies `base::c()` fallback to df-cols (#1462, #1640)", {
+  x <- structure(1, class = "myclass")
+  df <- tibble(a = tibble(x = x))
+  df <- vec_rbind(df, df)
+
+  expect_equal(df$a$x, structure(c(1, 1), class = "myclass"))
+})
+
 
 # Golden tests -------------------------------------------------------
 
@@ -1036,10 +1372,7 @@ test_that("row-binding performs expected allocations", {
     dfs <- map(dfs, set_rownames_recursively)
     with_memory_prof(vec_rbind_list(dfs))
 
-    # FIXME: The following recursive cases duplicate rownames
-    # excessively because df-cols are restored at each chunk
-    # assignment, causing a premature name-repair
-    "FIXME (#1217): Data frame with rownames (non-repaired, recursive case)"
+    "Data frame with rownames (non-repaired, recursive case) (#1217)"
     df <- data_frame(
       x = 1:2,
       y = data_frame(x = 1:2)
@@ -1048,8 +1381,20 @@ test_that("row-binding performs expected allocations", {
     dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
     with_memory_prof(vec_rbind_list(dfs))
 
-    "FIXME (#1217): Data frame with rownames (repaired, recursive case)"
+    "Data frame with rownames (repaired, recursive case) (#1217)"
     dfs <- map(dfs, set_rownames_recursively)
     with_memory_prof(vec_rbind_list(dfs))
   })
 })
+
+test_that("`.names_to` is assigned after restoration (#1648)", {
+  df <- data_frame(x = factor("foo"))
+  expect_equal(
+    vec_rbind(name = df, .names_to = "x"),
+    data_frame(x = "name")
+  )
+
+  # This used to fail with:
+  #> Error in `vctrs::vec_rbind()`:
+  #> ! adding class "factor" to an invalid object
+})
