@@ -1,4 +1,3 @@
-
 test_that("Casting to named argument mentions 'match type <foo>'", {
   expect_snapshot(error = TRUE, vec_cast(1, "", x_arg = "foo", to_arg = "bar"))
   expect_snapshot(error = TRUE, vec_cast(1, "", x_arg = "foo"))
@@ -19,15 +18,13 @@ test_that("casting requires vectors", {
   expect_error(vec_cast(quote(name), NULL), class = "vctrs_error_scalar_type")
   expect_error(vec_cast(quote(name), NA), class = "vctrs_error_scalar_type")
   expect_error(vec_cast(quote(name), list()), class = "vctrs_error_scalar_type")
-  expect_error(vec_cast(quote(name), quote(name)), class = "vctrs_error_scalar_type")
+  expect_error(
+    vec_cast(quote(name), quote(name)),
+    class = "vctrs_error_scalar_type"
+  )
 })
 
-test_that("casting between `NULL` and partial types is allowed", {
-  expect_identical(vec_cast(NULL, partial_factor()), NULL)
-  expect_identical(vec_cast(partial_factor(), NULL), partial_factor())
-})
-
-test_that("dimensionality matches output" ,{
+test_that("dimensionality matches output", {
   x1 <- matrix(TRUE, nrow = 1, ncol = 1)
   x2 <- matrix(1, nrow = 0, ncol = 2)
   expect_dim(vec_cast(x1, x2), c(1, 2))
@@ -119,14 +116,26 @@ test_that("can suppress cast errors selectively", {
   expect_error(regexp = NA, allow_lossy_cast(f()))
   expect_error(regexp = NA, allow_lossy_cast(f(), x_ptype = factor("a")))
   expect_error(regexp = NA, allow_lossy_cast(f(), to_ptype = factor("b")))
-  expect_error(regexp = NA, allow_lossy_cast(f(), x_ptype = factor("a"), to_ptype = factor("b")))
-  expect_error(allow_lossy_cast(f(), x_ptype = factor("c")), class = "vctrs_error_cast_lossy")
-  expect_error(allow_lossy_cast(f(), x_ptype = factor("b"), to_ptype = factor("a")), class = "vctrs_error_cast_lossy")
-  expect_error(allow_lossy_cast(f(), x_ptype = factor("a"), to_ptype = factor("c")), class = "vctrs_error_cast_lossy")
+  expect_error(
+    regexp = NA,
+    allow_lossy_cast(f(), x_ptype = factor("a"), to_ptype = factor("b"))
+  )
+  expect_error(
+    allow_lossy_cast(f(), x_ptype = factor("c")),
+    class = "vctrs_error_cast_lossy"
+  )
+  expect_error(
+    allow_lossy_cast(f(), x_ptype = factor("b"), to_ptype = factor("a")),
+    class = "vctrs_error_cast_lossy"
+  )
+  expect_error(
+    allow_lossy_cast(f(), x_ptype = factor("a"), to_ptype = factor("c")),
+    class = "vctrs_error_cast_lossy"
+  )
 })
 
 test_that("can signal deprecation warnings for lossy casts", {
-  local_lifecycle_warnings()
+  local_options(lifecycle_verbosity = "warning")
 
   lossy_cast <- function() {
     maybe_lossy_cast(
@@ -140,10 +149,19 @@ test_that("can signal deprecation warnings for lossy casts", {
     )
   }
 
-  expect_warning(expect_true(lossy_cast()), "detected a lossy transformation")
+  expect_snapshot({
+    (expect_warning(expect_true(lossy_cast())))
+  })
   expect_warning(regexp = NA, expect_true(allow_lossy_cast(lossy_cast())))
-  expect_warning(regexp = NA, expect_true(allow_lossy_cast(lossy_cast(), factor("foo"), factor("bar"))))
-  expect_warning(expect_true(allow_lossy_cast(lossy_cast(), factor("bar"), double())))
+  expect_warning(
+    regexp = NA,
+    expect_true(allow_lossy_cast(lossy_cast(), factor("foo"), factor("bar")))
+  )
+  expect_warning(expect_true(allow_lossy_cast(
+    lossy_cast(),
+    factor("bar"),
+    double()
+  )))
 })
 
 
@@ -157,38 +175,58 @@ test_that("vec_ptype_common() optionally falls back to base class", {
   y_df <- data_frame(x = y)
 
   expect_error(
-    vec_ptype_common_opts(x, y, .opts = full_fallback_opts()),
+    vec_ptype_common_params(x, y, .fallback_opts = enabled_fallback_opts()),
     class = "vctrs_error_incompatible_type"
   )
   expect_error(
-    vec_ptype_common_opts(x_df, y_df, .opts = full_fallback_opts()),
+    vec_ptype_common_params(
+      x_df,
+      y_df,
+      .fallback_opts = enabled_fallback_opts()
+    ),
     class = "vctrs_error_incompatible_type"
   )
 
   expect_error(
-    vec_cast_common_opts(x, y, .opts = full_fallback_opts()),
+    vec_ptype_common_params(x, y, .fallback_opts = enabled_fallback_opts()),
     class = "vctrs_error_incompatible_type"
   )
   expect_error(
-    vec_cast_common_opts(x_df, y_df, .opts = full_fallback_opts()),
+    vec_ptype_common_params(
+      x_df,
+      y_df,
+      .fallback_opts = enabled_fallback_opts()
+    ),
     class = "vctrs_error_incompatible_type"
   )
 
   class(y) <- c("foo", class(x))
   y_df <- data_frame(x = y)
 
-  common_sentinel <- vec_ptype_common_opts(x, y, .opts = full_fallback_opts())
+  common_sentinel <- vec_ptype_common_params(
+    x,
+    y,
+    .fallback_opts = enabled_fallback_opts()
+  )
   expect_true(is_common_class_fallback(common_sentinel))
   expect_identical(fallback_class(common_sentinel), "vctrs_foobar")
 
-  common_sentinel <- vec_ptype_common_opts(x_df, y_df, .opts = full_fallback_opts())
+  common_sentinel <- vec_ptype_common_params(
+    x_df,
+    y_df,
+    .fallback_opts = enabled_fallback_opts()
+  )
   expect_true(is_common_class_fallback(common_sentinel$x))
   expect_identical(fallback_class(common_sentinel$x), "vctrs_foobar")
 
-  common <- vec_cast_common_opts(x = x, y = y, .opts = full_fallback_opts())
+  common <- vec_cast_common_opts(x = x, y = y, .opts = enabled_fallback_opts())
   expect_identical(common, list(x = x, y = y))
 
-  common <- vec_cast_common_opts(x = x_df, y = y_df, .opts = full_fallback_opts())
+  common <- vec_cast_common_opts(
+    x = x_df,
+    y = y_df,
+    .opts = enabled_fallback_opts()
+  )
   expect_identical(common, list(x = x_df, y = y_df))
 })
 
@@ -299,3 +337,38 @@ test_that("bare-type fallback for df-cast works", {
 
   expect_error(vec_rbind(gdf, gdf), NA)
 })
+
+test_that("can cast to unspecified `NA` with `vec_cast()` and `vec_cast_common()` (#2099)", {
+  # In the `vec_cast()` case no `vec_ptype()` call is made, which means that no
+  # finalization step is required. In the `vec_cast_common()` case, the
+  # underlying call to `vec_ptype_common()` calls `vec_ptype(NA)` but also
+  # finalizes that to `logical()` on the way out, so this still works.
+  expect_identical(vec_cast(TRUE, to = NA), TRUE)
+  expect_identical(vec_cast_common(TRUE, .to = NA), list(TRUE))
+
+  # `vec_cast()` itself does not call `vec_ptype()` and does not finalize,
+  # so this stays <unspecified> and the cast fails
+  # (this behavior is questionable, but is very much an edge case)
+  expect_snapshot(error = TRUE, {
+    vec_cast(TRUE, to = unspecified(1))
+  })
+
+  # `vec_cast_common()` calls `vec_ptype_common()`, which always finalises,
+  # so this technically works (but again, it is an edge case)
+  expect_identical(vec_cast_common(TRUE, .to = unspecified(1)), list(TRUE))
+})
+
+# Golden tests -------------------------------------------------------
+
+test_that("casting performs expected allocations", {
+  expect_snapshot({
+    # No allocations when shape doesn't change (#2006)
+    x <- matrix(rep(1L, 1e2), ncol = 2)
+    with_memory_prof(vec_cast(x, x))
+
+    # One allocation when type changes
+    x <- matrix(rep(1L, 1e2), ncol = 2)
+    y <- matrix(rep(1, 1e2), ncol = 2)
+    with_memory_prof(vec_cast(x, y))
+  })
+})
