@@ -1,6 +1,5 @@
-
 test_that("vec_as_index() still works", {
-  local_lifecycle_silence()
+  local_options(lifecycle_verbosity = "quiet")
   expect_identical(vec_as_index(-2, 10), vec_as_location(-2, 10))
   expect_identical(
     vec_as_index("cyl", length(mtcars), names(mtcars)),
@@ -9,7 +8,7 @@ test_that("vec_as_index() still works", {
 })
 
 test_that("vec_repeat() still works", {
-  local_lifecycle_silence()
+  local_options(lifecycle_verbosity = "quiet")
   expect_identical(vec_repeat(1:2, times = 2), vec_rep(1:2, 2))
   expect_identical(vec_repeat(1:2, each = 2), vec_rep_each(1:2, 2))
 })
@@ -39,3 +38,19 @@ test_that("vec_equal_na() still works", {
     c(FALSE, TRUE, FALSE, TRUE)
   )
 })
+
+test_that("vec_is_list() still works", {
+  expect_false(vec_is_list(1))
+  expect_true(vec_is_list(list()))
+})
+
+test_that("vec_check_list() still works", {
+  my_check <- function(x) vec_check_list(x)
+
+  expect_snapshot(error = TRUE, {
+    vec_check_list(1)
+  })
+  expect_snapshot(error = TRUE, {
+    my_check(1)
+  })
+})
