@@ -1,12 +1,17 @@
-
 test_that("vec_slice throws error with non-vector inputs", {
   expect_error(vec_slice(environment(), 1L), class = "vctrs_error_scalar_type")
 })
 
 test_that("vec_slice throws error with non-vector subscripts", {
   expect_snapshot({
-    (expect_error(vec_slice(1:3, Sys.Date()), class = "vctrs_error_subscript_type"))
-    (expect_error(vec_slice(1:3, matrix(TRUE, nrow = 1)), class = "vctrs_error_subscript_type"))
+    (expect_error(
+      vec_slice(1:3, Sys.Date()),
+      class = "vctrs_error_subscript_type"
+    ))
+    (expect_error(
+      vec_slice(1:3, matrix(TRUE, nrow = 1)),
+      class = "vctrs_error_subscript_type"
+    ))
   })
 })
 
@@ -38,7 +43,7 @@ test_that("can subset with missing indices", {
     expect_identical(vec_slice(lgl(1, 0, 1), i), lgl(0, NA))
     expect_identical(vec_slice(int(1, 2, 3), i), int(2, NA))
     expect_identical(vec_slice(dbl(1, 2, 3), i), dbl(2, NA))
-    expect_identical(vec_slice(cpl(1, 2, 3), i), cpl(2, NA))
+    expect_identical(vec_slice(cpl2(1, 2, 3), i), cpl2(2, NA))
     expect_identical(vec_slice(chr("1", "2", "3"), i), c("2", NA))
     expect_identical(vec_slice(raw2(1, 2, 3), i), raw2(2, 0))
     expect_identical(vec_slice(list(1, 2, 3), i), list(2, NULL))
@@ -53,7 +58,10 @@ test_that("can subset with a recycled NA", {
 
   rownames <- rep_len("", nrow(mtcars))
   rownames <- vec_as_names(rownames, repair = "unique")
-  expect_identical(vec_slice(mtcars, NA), structure(mtcars[NA, ], row.names = rownames))
+  expect_identical(
+    vec_slice(mtcars, NA),
+    structure(mtcars[NA, ], row.names = rownames)
+  )
 })
 
 test_that("can subset with a recycled TRUE", {
@@ -77,7 +85,10 @@ test_that("can't index beyond the end of a vector", {
 })
 
 test_that("slicing non existing elements fails", {
-  expect_error(vec_as_location("foo", 1L, "f"), class = "vctrs_error_subscript_oob")
+  expect_error(
+    vec_as_location("foo", 1L, "f"),
+    class = "vctrs_error_subscript_oob"
+  )
   expect_error(vec_slice(c(f = 1), "foo"), class = "vctrs_error_subscript_oob")
 })
 
@@ -183,7 +194,12 @@ test_that("0 is ignored in positive indices", {
 
 test_that("can slice with double indices", {
   expect_identical(vec_slice(1:3, dbl(2, 3)), 2:3)
-  expect_snapshot((expect_error(vec_as_location(2^31, 3L), class = "vctrs_error_subscript_type")))
+  expect_snapshot(
+    (expect_error(
+      vec_as_location(2^31, 3L),
+      class = "vctrs_error_subscript_type"
+    ))
+  )
 })
 
 test_that("can slice with symbols", {
@@ -227,11 +243,10 @@ test_that("dimensions are preserved by vec_slice()", {
   exp <- foobar(
     c(1L, 3L),
     dim = c(1, 2),
-    dimnames = list(a = "foo", b = c("quux", "hunoz")
-  ))
+    dimnames = list(a = "foo", b = c("quux", "hunoz"))
+  )
   expect_identical(out, exp)
 
-
   # Native case
   attrib <- NULL
 
@@ -300,7 +315,10 @@ test_that("vec_slice() falls back to `[` with S3 objects", {
   )
   expect_identical(vec_slice(foobar(NA), 1), "dispatched")
 
-  expect_error(vec_slice(foobar(list(NA)), 1), class = "vctrs_error_scalar_type")
+  expect_error(
+    vec_slice(foobar(list(NA)), 1),
+    class = "vctrs_error_scalar_type"
+  )
   local_methods(
     vec_proxy.vctrs_foobar = function(x, ...) x
   )
@@ -402,7 +420,10 @@ test_that("can slice with numerics (#577)", {
 test_that("missing indices don't create NA names", {
   x <- set_names(letters)
   expect_identical(vec_slice(x, na_int), set_names(na_chr, ""))
-  expect_identical(vec_slice(x, int(1, NA, 3, NA)), chr(a = "a", NA, c = "c", NA))
+  expect_identical(
+    vec_slice(x, int(1, NA, 3, NA)),
+    chr(a = "a", NA, c = "c", NA)
+  )
 
   # Preserves existing NA names
   x <- set_names(1:2, c(NA, "foo"))
@@ -425,12 +446,53 @@ test_that("slicing an unspecified() object returns an unspecified()", {
 
 
 test_that("vec_slice() works with Altrep classes with custom extract methods", {
-  skip_if(getRversion() < "3.5")
-
-  x <- .Call(vctrs_altrep_rle_Make, c(foo = 10L, bar = 5L))
+  x <- chr_rle(foo = 10L, bar = 5L)
+  expect_false(chr_rle_is_materialized(x))
 
   idx <- c(9, 10, 11)
   expect_equal(vec_slice(x, idx), c("foo", "foo", "bar"))
+  expect_false(chr_rle_is_materialized(x))
+
+  # With zero
+  idx <- c(0, 1)
+  expect_equal(vec_slice(x, idx), "foo")
+  expect_false(chr_rle_is_materialized(x))
+
+  # With integer missing values
+  idx <- c(0, NA, 2, NA)
+  expect_equal(vec_slice(x, idx), c(NA, "foo", NA))
+  expect_false(chr_rle_is_materialized(x))
+
+  # With logical condition index
+  idx <- c(TRUE, rep(FALSE, 8), TRUE, rep(FALSE, 2), TRUE, TRUE, TRUE)
+  expect_equal(vec_slice(x, idx), c("foo", "foo", "bar", "bar", "bar"))
+  expect_false(chr_rle_is_materialized(x))
+
+  # Everything
+  idx <- TRUE
+  expect_equal(vec_slice(x, idx), c(rep("foo", 10), rep("bar", 5)))
+  expect_false(chr_rle_is_materialized(x))
+
+  # Nothing
+  idx <- FALSE
+  expect_equal(vec_slice(x, idx), character())
+  expect_false(chr_rle_is_materialized(x))
+
+  # Whole vector of missing values
+  idx <- NA
+  expect_equal(vec_slice(x, idx), rep(NA_character_, 15))
+  expect_false(chr_rle_is_materialized(x))
+
+  # Just 1 missing value
+  idx <- NA_integer_
+  expect_equal(vec_slice(x, idx), rep(NA_character_, 1))
+  expect_false(chr_rle_is_materialized(x))
+
+  # OOB
+  idx <- 16
+  expect_snapshot(error = TRUE, {
+    vec_slice(x, idx)
+  })
 })
 
 test_that("Unnamed vector with character subscript is caught", {
@@ -498,11 +560,10 @@ test_that("vec_init() asserts vectorness (#301)", {
 })
 
 test_that("vec_init() works with Altrep classes", {
-  skip_if(getRversion() < "3.5")
-
-  x <- .Call(vctrs_altrep_rle_Make, c(foo = 1L, bar = 2L))
-
+  x <- chr_rle(foo = 1L, bar = 2L)
+  expect_false(chr_rle_is_materialized(x))
   expect_equal(vec_init(x, 2), rep(NA_character_, 2))
+  expect_false(chr_rle_is_materialized(x))
 })
 
 test_that("vec_init() validates `n`", {
@@ -532,11 +593,10 @@ test_that("names are recycled correctly with compact reps", {
 })
 
 test_that("vec_slice() with compact_reps work with Altrep classes", {
-  skip_if(getRversion() < "3.5")
-
-  x <- .Call(vctrs_altrep_rle_Make, c(foo = 10L, bar = 5L))
-
+  x <- chr_rle(foo = 10L, bar = 5L)
+  expect_false(chr_rle_is_materialized(x))
   expect_equal(vec_slice_rep(x, 10L, 3L), rep("foo", 3))
+  expect_false(chr_rle_is_materialized(x))
 })
 
 # vec_slice + compact_seq -------------------------------------------------
@@ -547,26 +607,68 @@ test_that("can subset base vectors with compact seqs", {
   start <- 1L
   size <- 2L
   increasing <- TRUE
-  expect_identical(vec_slice_seq(lgl(1, 0, 1), start, size, increasing), lgl(0, 1))
-  expect_identical(vec_slice_seq(int(1, 2, 3), start, size, increasing), int(2, 3))
-  expect_identical(vec_slice_seq(dbl(1, 2, 3), start, size, increasing), dbl(2, 3))
-  expect_identical(vec_slice_seq(cpl(1, 2, 3), start, size, increasing), cpl(2, 3))
-  expect_identical(vec_slice_seq(chr("1", "2", "3"), start, size, increasing), chr("2", "3"))
-  expect_identical(vec_slice_seq(raw2(1, 2, 3), start, size, increasing), raw2(2, 3))
-  expect_identical(vec_slice_seq(list(1, 2, 3), start, size, increasing), list(2, 3))
+  expect_identical(
+    vec_slice_seq(lgl(1, 0, 1), start, size, increasing),
+    lgl(0, 1)
+  )
+  expect_identical(
+    vec_slice_seq(int(1, 2, 3), start, size, increasing),
+    int(2, 3)
+  )
+  expect_identical(
+    vec_slice_seq(dbl(1, 2, 3), start, size, increasing),
+    dbl(2, 3)
+  )
+  expect_identical(
+    vec_slice_seq(cpl(1, 2, 3), start, size, increasing),
+    cpl(2, 3)
+  )
+  expect_identical(
+    vec_slice_seq(chr("1", "2", "3"), start, size, increasing),
+    chr("2", "3")
+  )
+  expect_identical(
+    vec_slice_seq(raw2(1, 2, 3), start, size, increasing),
+    raw2(2, 3)
+  )
+  expect_identical(
+    vec_slice_seq(list(1, 2, 3), start, size, increasing),
+    list(2, 3)
+  )
 })
 
 test_that("can subset base vectors with decreasing compact seqs", {
   start <- 2L
   size <- 2L
   increasing <- FALSE
-  expect_identical(vec_slice_seq(lgl(1, 0, 1), start, size, increasing), lgl(1, 0))
-  expect_identical(vec_slice_seq(int(1, 2, 3), start, size, increasing), int(3, 2))
-  expect_identical(vec_slice_seq(dbl(1, 2, 3), start, size, increasing), dbl(3, 2))
-  expect_identical(vec_slice_seq(cpl(1, 2, 3), start, size, increasing), cpl(3, 2))
-  expect_identical(vec_slice_seq(chr("1", "2", "3"), start, size, increasing), chr("3", "2"))
-  expect_identical(vec_slice_seq(raw2(1, 2, 3), start, size, increasing), raw2(3, 2))
-  expect_identical(vec_slice_seq(list(1, 2, 3), start, size, increasing), list(3, 2))
+  expect_identical(
+    vec_slice_seq(lgl(1, 0, 1), start, size, increasing),
+    lgl(1, 0)
+  )
+  expect_identical(
+    vec_slice_seq(int(1, 2, 3), start, size, increasing),
+    int(3, 2)
+  )
+  expect_identical(
+    vec_slice_seq(dbl(1, 2, 3), start, size, increasing),
+    dbl(3, 2)
+  )
+  expect_identical(
+    vec_slice_seq(cpl(1, 2, 3), start, size, increasing),
+    cpl(3, 2)
+  )
+  expect_identical(
+    vec_slice_seq(chr("1", "2", "3"), start, size, increasing),
+    chr("3", "2")
+  )
+  expect_identical(
+    vec_slice_seq(raw2(1, 2, 3), start, size, increasing),
+    raw2(3, 2)
+  )
+  expect_identical(
+    vec_slice_seq(list(1, 2, 3), start, size, increasing),
+    list(3, 2)
+  )
 })
 
 test_that("can subset base vectors with size 0 compact seqs", {
@@ -577,9 +679,18 @@ test_that("can subset base vectors with size 0 compact seqs", {
   expect_identical(vec_slice_seq(int(1, 2, 3), start, size, increasing), int())
   expect_identical(vec_slice_seq(dbl(1, 2, 3), start, size, increasing), dbl())
   expect_identical(vec_slice_seq(cpl(1, 2, 3), start, size, increasing), cpl())
-  expect_identical(vec_slice_seq(chr("1", "2", "3"), start, size, increasing), chr())
-  expect_identical(vec_slice_seq(raw2(1, 2, 3), start, size, increasing), raw2())
-  expect_identical(vec_slice_seq(list(1, 2, 3), start, size, increasing), list())
+  expect_identical(
+    vec_slice_seq(chr("1", "2", "3"), start, size, increasing),
+    chr()
+  )
+  expect_identical(
+    vec_slice_seq(raw2(1, 2, 3), start, size, increasing),
+    raw2()
+  )
+  expect_identical(
+    vec_slice_seq(list(1, 2, 3), start, size, increasing),
+    list()
+  )
 })
 
 test_that("can subset shaped base vectors with compact seqs", {
@@ -587,13 +698,34 @@ test_that("can subset shaped base vectors with compact seqs", {
   size <- 2L
   increasing <- TRUE
   mat <- as.matrix
-  expect_identical(vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing), mat(lgl(0, 1)))
-  expect_identical(vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing), mat(int(2, 3)))
-  expect_identical(vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing), mat(dbl(2, 3)))
-  expect_identical(vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing), mat(cpl(2, 3)))
-  expect_identical(vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing), mat(chr("2", "3")))
-  expect_identical(vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing), mat(raw2(2, 3)))
-  expect_identical(vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing), mat(list(2, 3)))
+  expect_identical(
+    vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing),
+    mat(lgl(0, 1))
+  )
+  expect_identical(
+    vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing),
+    mat(int(2, 3))
+  )
+  expect_identical(
+    vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing),
+    mat(dbl(2, 3))
+  )
+  expect_identical(
+    vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing),
+    mat(cpl(2, 3))
+  )
+  expect_identical(
+    vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing),
+    mat(chr("2", "3"))
+  )
+  expect_identical(
+    vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing),
+    mat(raw2(2, 3))
+  )
+  expect_identical(
+    vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing),
+    mat(list(2, 3))
+  )
 })
 
 test_that("can subset shaped base vectors with decreasing compact seqs", {
@@ -601,13 +733,34 @@ test_that("can subset shaped base vectors with decreasing compact seqs", {
   size <- 2L
   increasing <- FALSE
   mat <- as.matrix
-  expect_identical(vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing), mat(lgl(1, 0)))
-  expect_identical(vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing), mat(int(3, 2)))
-  expect_identical(vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing), mat(dbl(3, 2)))
-  expect_identical(vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing), mat(cpl(3, 2)))
-  expect_identical(vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing), mat(chr("3", "2")))
-  expect_identical(vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing), mat(raw2(3, 2)))
-  expect_identical(vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing), mat(list(3, 2)))
+  expect_identical(
+    vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing),
+    mat(lgl(1, 0))
+  )
+  expect_identical(
+    vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing),
+    mat(int(3, 2))
+  )
+  expect_identical(
+    vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing),
+    mat(dbl(3, 2))
+  )
+  expect_identical(
+    vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing),
+    mat(cpl(3, 2))
+  )
+  expect_identical(
+    vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing),
+    mat(chr("3", "2"))
+  )
+  expect_identical(
+    vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing),
+    mat(raw2(3, 2))
+  )
+  expect_identical(
+    vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing),
+    mat(list(3, 2))
+  )
 })
 
 test_that("can subset shaped base vectors with size 0 compact seqs", {
@@ -615,13 +768,34 @@ test_that("can subset shaped base vectors with size 0 compact seqs", {
   size <- 0L
   increasing <- TRUE
   mat <- as.matrix
-  expect_identical(vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing), mat(lgl()))
-  expect_identical(vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing), mat(int()))
-  expect_identical(vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing), mat(dbl()))
-  expect_identical(vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing), mat(cpl()))
-  expect_identical(vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing), mat(chr()))
-  expect_identical(vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing), mat(raw2()))
-  expect_identical(vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing), mat(list()))
+  expect_identical(
+    vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing),
+    mat(lgl())
+  )
+  expect_identical(
+    vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing),
+    mat(int())
+  )
+  expect_identical(
+    vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing),
+    mat(dbl())
+  )
+  expect_identical(
+    vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing),
+    mat(cpl())
+  )
+  expect_identical(
+    vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing),
+    mat(chr())
+  )
+  expect_identical(
+    vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing),
+    mat(raw2())
+  )
+  expect_identical(
+    vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing),
+    mat(list())
+  )
 })
 
 test_that("can subset object of any dimensionality with compact seqs", {
@@ -661,17 +835,19 @@ test_that("can subset S3 objects using the fallback method with compact seqs", {
 })
 
 test_that("vec_slice() with compact_seqs work with Altrep classes", {
-  skip_if(getRversion() < "3.5")
-
-  x <- .Call(vctrs_altrep_rle_Make, c(foo = 2L, bar = 3L))
-
+  x <- chr_rle(foo = 2L, bar = 3L)
+  expect_false(chr_rle_is_materialized(x))
   expect_equal(vec_slice_seq(x, 1L, 3L), c("foo", "bar", "bar"))
+  expect_false(chr_rle_is_materialized(x))
 })
 
 test_that("vec_slice() handles symbols and OO objects", {
   expect_identical(vec_slice(c(a = 1, b = 2), quote(b)), c(b = 2))
   expect_identical(vec_slice(c(a = 1, b = 2), factor("b")), c(b = 2))
-  expect_error(vec_slice(c(a = 1, b = 2), foobar("b")), class = "vctrs_error_subscript_type")
+  expect_error(
+    vec_slice(c(a = 1, b = 2), foobar("b")),
+    class = "vctrs_error_subscript_type"
+  )
 })
 
 test_that("vec_init() handles names in columns", {
